import cherrypy
from Cheetah.Template import Template
import dailyspents.util as util
from GenericView import GenericView
from dailyspents.model.Spent import Spent
from dailyspents.model.Payment import Payment
from dailyspents.model.Category import Category
from dailyspents.model.Local import Local
import gdchart
from dailyspents.csvUtil import importCSV
import os

class SpentView(GenericView):
    
    def __init__(self):
        GenericView.__init__(self)
    
    @cherrypy.expose
    def index(self, message=None, errorMessage=None, page='newspent', **kargs):
        
        GenericView.index(self, message, errorMessage)
        
        realPage = ''
        
        if page == 'newspent':
            realPage = "insertSpent.html"
        elif page == 'filter':
            realPage = "filter.html"
        elif page == 'edit':
            return self.prepareEdit(int(kargs['id']))
        elif page == 'importCSV':
            realPage = 'importCSV.html'
        else:
            raise cherrypy.HTTPRedirect('/index')
            
        return Template(file=util.HTMLPATH + realPage,
                        searchList=[self.nameSpace]).respond()
    
    @cherrypy.expose
    def newPayment(self, **kargs):
        
        paymentName = kargs.get('name', '')
        
        try:
            if paymentName == '':
                raise
            payment = Payment()
            payment.name = paymentName
            payment.user = self.user
            self.paymentController.save(payment)
            self.message = "New payment inserted sucefully."
        except Exception, e:
            print '[an exception occurred] >>>', e
            self.message = 'Invalid name.'
            raise cherrypy.HTTPRedirect("/spent?errorMessage="+self.message)
        
        raise cherrypy.HTTPRedirect("/spent?message="+self.message)
    
    @cherrypy.expose
    def newCategory(self, **kargs):
        
        categoryName = kargs.get('name', '')
        
        try:
            if categoryName == '':
                raise
            category = Category()
            category.name = categoryName
            category.user = self.user
            self.categoryController.save(category)
            self.message = "New category inserted sucefully."
        except Exception, e:
            print '[an exception occurred] >>>', e
            self.message = 'Invalid name.'
            raise cherrypy.HTTPRedirect("/spent?errorMessage="+self.message)
        
        raise cherrypy.HTTPRedirect("/spent?message="+self.message)
    
    @cherrypy.expose
    def newLocal(self, **kargs):
        newLocalName = kargs.get('name', None)
        
        try:
            
            if newLocalName == None or newLocalName == '':
                raise Exception('Invalid new local name.')
            
            newLocal = Local(name=newLocalName, user=self.user)
            self.localController.save(newLocal)
            self.message = 'New local inserted sucefully.'
            
        except Exception, e:
            print '[an exception occurred] >>>', e
            self.message = 'Invalid new local name.'
            raise cherrypy.HTTPRedirect("/spent?page=newspent&errorMessage="+
                                        self.message)
        
        raise cherrypy.HTTPRedirect("/spent/index?message="+self.message)
    
    def prepareEdit(self, id, errorMessage=None, message=None):
        try:
            spent = self.spentController.getById(id)
            self.nameSpace['spent'] = spent
            self.nameSpace['errorMessage'] = errorMessage
            msg = self.nameSpace['message'] = message
            
            return Template(file=util.HTMLPATH+"editSpent.html", 
                        searchList=[self.nameSpace]).respond()
        except Exception, e:
            raise e
    
    @cherrypy.expose
    def edit(self, **kargs):
        
        self.checkAuthentication()
        
        try:
            spent = self.spentController.getById(int(kargs['id']))
            self.fillSpent(spent, **kargs)
            self.spentController.save(spent)
            
            return self.prepareEdit(spent.id, message = 'Item edited sucefully.')
        except Exception, e:
            print '[an exception occurred] >>>', e
            return self.prepareEdit(spent.id,
                            errorMessage = 'Invalid inserted data.')
        
    @cherrypy.expose                    
    def insertSpent(self, **kargs):
        self.checkAuthentication()
        
        try:

            newSpent = Spent()
            newSpent.user = self.user
            self.fillSpent(newSpent, **kargs)
            
            self.spentController.save(newSpent)
            self.message = "New spent sucefully inserted."
            
        except Exception, e:
            self.message = 'Invalid inserted data.'
            raise cherrypy.HTTPRedirect("/spent/index?errorMessage="+self.message)
        
        raise cherrypy.HTTPRedirect("/spent/index?message="+self.message)
    
    def fillSpent(self, spent, **kargs):
        '''Fills a spent object with posted params'''
        
        categoryId = kargs.get('categoryId')
        paymentId = kargs.get('paymentId')
        localId = kargs.get('localId')
        
        if categoryId == '':
            categoryId = 0
            
        if paymentId == '':
            paymentId = 0
            
        if localId == '':
            localId = 0
        
        cat = self.categoryController.getById(int(categoryId))
        spent.category = cat
        
        payment = self.paymentController.getById(int(paymentId))
        spent.payment = payment
        
        local = self.localController.getById(int(localId))
        spent.local = local
        
        spent.value = eval(kargs['value'])
        spent.description = kargs['description']
        spent.date = kargs['date']
                 
    @cherrypy.expose
    def filter(self, **kargs):
        
        self.checkAuthentication()
        
        self.nameSpace = self.makeCommonNamespace()
        self.makeContextMenu()
        self.nameSpace['contextMenu'].append(('Show statistics', 
                '/spent/makeStatistics'))
        self.nameSpace['doNavigation'] = False
        
        dateMin = kargs.get('from', None)
        dateMax = kargs.get('to', None)
        value = kargs.get('value', None)
        cat = kargs.get('categoryId', None)
        pay = kargs.get('paymentId', None)
        local = kargs.get('localId', None)
        
        if cat == "": cat = None
        
        if pay == '': pay = None
        
        if local == '': local = None
            
        if dateMin == "": dateMin = None
            
        if dateMax == "": dateMax = None
        
        if value == "": value = None
        
        spents = self.spentController.filter((dateMin, dateMax), value, cat,
            pay, local)
        
        self.nameSpace['spents'] = spents
        cherrypy.session['spents'] = spents
        
        if int(kargs.get('showStatistics', 0)) == 1:
            return self.makeStatistics()
        
        return Template(file=util.HTMLPATH+"main.html",
                        searchList=[self.nameSpace]).respond()
                        
    @cherrypy.expose
    def makeStatistics(self, **kargs):
        
        self.checkAuthentication()
        
        total = 0
        
        #a dict mapping each category to a tuple with its total and percentage
        #eg.: {'cat': (10, 23.5)}
        totalPerCategory = {}
        
        #a dict mapping each payment to a tuple with its total and percentage
        #eg.: {'pay': (10, 23.5)}
        totalPerPayment = {}
        
        spents = cherrypy.session.get('spents', [])
        
        for spent in spents:
            total += spent.value
            
            catAtualValue = spent.value + totalPerCategory.get(
                    spent.category.name, (0,0))[0]
                    
            totalPerCategory[spent.category.name] = (catAtualValue, 
                    (catAtualValue*100)/total)
            
            payAtualValue = spent.value + totalPerPayment.get(
                    spent.payment.name, (0,0))[0]
                        
            totalPerPayment[spent.payment.name] = (payAtualValue, 
                    (payAtualValue*100)/total)
        
        #create the cateogry chart
        chartCategory = gdchart.Bar3D()
        
        chartCategory.setData(list(value[0] for value in totalPerCategory.values()))
        chartCategory.setLabels(list(totalPerCategory.keys()))
        chartCategory.bg_color = gdchart.rgbFactory('white')
        chartCategory.width = 350
        chartCategory.height = 300
        
        chartCategory.ext_color = util.COLORS[:len(totalPerCategory.keys())]
        
        chartCategory.draw(util.WEBPATH + 
                'tmp' + os.sep + 'category' + cherrypy.session['_id'] + '.png')
        
        #create the payment chart
        payCategory = gdchart.Bar3D()
        
        payCategory.setData(list(value[0] for value in totalPerPayment.values()))
        payCategory.setLabels(list(totalPerPayment.keys()))
        payCategory.bg_color = gdchart.rgbFactory('white')
        payCategory.width = 350
        payCategory.height = 300
        
        payCategory.ext_color = util.COLORS[:len(totalPerPayment.keys())]
        
        payCategory.draw(util.WEBPATH + 
                'tmp' + os.sep + 'payment' + cherrypy.session['_id'] + '.png')
        
        #configure the specific nameSpace
        self.nameSpace = self.makeCommonNamespace()
        self.nameSpace['totalPerCategory'] = totalPerCategory
        self.nameSpace['totalPerPayment'] = totalPerPayment
        self.nameSpace['total'] = total
        
        self.makeContextMenu()
        
        return Template(file=util.HTMLPATH+"showStatistics.html",
                        searchList=[self.nameSpace]).respond()
    
    @cherrypy.expose
    def commaSeparatedVersion(self):
        
        self.checkAuthentication()
        
        outputText = ''
        
        for s in cherrypy.session['spents']:
            line = (s.category.name, s.payment.name, str(s.date), 
                    s.description, str(s.value))
            outputText += ','.join(line) + '\n'
        
        fileName = util.WEBPATH + 'tmp' + os.sep + 'csv' + \
                        cherrypy.session['_id'] + '.txt'
                        
        fileOutput = open(fileName, 'w')
        fileOutput.write(outputText)
        fileOutput.close()
        
        return file(fileName)

    @cherrypy.expose
    def postCSVFile(self, csvFile):
        
        self.checkAuthentication()
        
        fileName = util.WEBPATH + 'tmp' + os.sep + 'csvSource' + \
                        cherrypy.session['_id'] + '.txt'
        fileSource = open(fileName, 'w')
        fileSource.write(str(csvFile).encode('utf-8'))
        fileSource.close()
        
        importCSV(fileName, cherrypy.session['user'])
        
        os.remove(fileName)
        
        self.makeCommonNamespace()
        self.makeContextMenu()
        self.message = 'CSV imported sucefully'
        
        raise cherrypy.HTTPRedirect("/index?page=importCSV&message=" +
                                        self.message)